package Alt::Sub::Delete::NewPackageSeparator;
use base qw(Alt::Base);

use strict;

our $VERSION = '1.000021'; # VERSION
our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2023-12-01'; # DATE
our $DIST = 'Alt-Sub-Delete-NewPackageSeparator'; # DIST

1;
# ABSTRACT: Alternate implementation of Sub::Delete which replaces the old package separator with new

__END__

=pod

=encoding UTF-8

=head1 NAME

Alt::Sub::Delete::NewPackageSeparator - Alternate implementation of Sub::Delete which replaces the old package separator with new

=head1 VERSION

This document describes version 1.000021 of Alt::Sub::Delete::NewPackageSeparator (from Perl distribution Alt-Sub-Delete-NewPackageSeparator), released on 2023-12-01.

=head1 DESCRIPTION

As of this writing, Sub::Delete (v1.00002) still uses the old package separator
(C<'> instead of C<::>) which triggers a deprecation warning under perl 5.38+.
This distribution includes an alternate Sub::Delete which replaces the use of
old package separator with new.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Alt-Sub-Delete-NewPackageSeparator>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Alt-Sub-Delete-NewPackageSeparator>.

=head1 SEE ALSO

L<Alt>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTING


To contribute, you can send patches by email/via RT, or send pull requests on
GitHub.

Most of the time, you don't need to build the distribution yourself. You can
simply modify the code, then test via:

 % prove -l

If you want to build the distribution (e.g. to try to install it locally on your
system), you can install L<Dist::Zilla>,
L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
that are considered a bug and can be reported to me.

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2023 by perlancar <perlancar@cpan.org>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Alt-Sub-Delete-NewPackageSeparator>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=cut
